@@ -23,9 +23,9 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <libdivecomputer/oceanic_atom2.h>
 #include <libdivecomputer/units.h>
 
+#include "oceanic_atom2.h"
 #include "oceanic_common.h"
 #include "context-private.h"
 #include "parser-private.h"
@@ -77,6 +77,7 @@
 #define OCI         0x454B
 #define A300CS      0x454C
 #define MUNDIAL3    0x4550
+#define PROPLUSX    0x4552
 #define F10B        0x4553
 #define F11B        0x4554
 #define XPAIR       0x4555
@@ -86,6 +87,7 @@
 #define I750TC      0x455A
 #define I450T       0x4641
 #define I550        0x4642
+#define I200        0x4646
 
 #define NORMAL   0
 #define GAUGE    1
@@ -157,7 +159,8 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		model == OCS || model == PROPLUS3 ||
 		model == A300 || model == MANTA ||
 		model == INSIGHT2 || model == ZEN ||
-		model == I300 || model == I550) {
+		model == I300 || model == I550 ||
+		model == I200) {
 		parser->headersize -= PAGESIZE;
 	} else if (model == VT4 || model == VT41) {
 		parser->headersize += PAGESIZE;
@@ -175,6 +178,8 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 	} else if (model == A300CS || model == VTX ||
 		model == I450T || model == I750TC) {
 		parser->headersize = 5 * PAGESIZE;
+	} else if (model == PROPLUSX) {
+		parser->headersize = 3 * PAGESIZE;
 	}
 
 	parser->serial = serial;
@@ -270,6 +275,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case COMPUMASK:
 		case INSIGHT2:
 		case I300:
+		case I200:
 			datetime->year   = ((p[3] & 0xE0) >> 1) + (p[4] & 0x0F) + 2000;
 			datetime->month  = (p[4] & 0xF0) >> 4;
 			datetime->day    = p[3] & 0x1F;
@@ -310,6 +316,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case VTX:
 		case I450T:
 		case I750TC:
+		case PROPLUSX:
 			datetime->year   = (p[10]) + 2000;
 			datetime->month  = (p[8]);
 			datetime->day    = (p[9]);
@@ -325,6 +332,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 			break;
 		}
 		datetime->second = 0;
+		datetime->timezone = DC_TIMEZONE_NONE;
 
 		// Convert to a 24-hour clock.
 		datetime->hour %= 12;
@@ -402,7 +410,8 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	} else if (parser->model == T3B || parser->model == VT3 ||
 		parser->model == DG03) {
 		mode = (data[2] & 0xC0) >> 6;
-	} else if (parser->model == VEO20 || parser->model == VEO30) {
+	} else if (parser->model == VEO20 || parser->model == VEO30 ||
+		parser->model == OCS) {
 		mode = (data[1] & 0x60) >> 5;
 	}
 
@@ -636,7 +645,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 	if (parser->mode != FREEDIVE) {
 		unsigned int idx = 0x17;
 		if (parser->model == A300CS || parser->model == VTX ||
-			parser->model == I450T || parser->model == I750TC)
+			parser->model == I450T || parser->model == I750TC ||
+			parser->model == PROPLUSX)
 			idx = 0x1f;
 		switch (data[idx] & 0x03) {
 		case 0:
@@ -691,7 +701,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == OC1C || parser->model == OCI ||
 		parser->model == TX1 || parser->model == A300CS ||
 		parser->model == VTX || parser->model == I450T ||
-		parser->model == I750TC) {
+		parser->model == I750TC || parser->model == PROPLUSX) {
 		samplesize = PAGESIZE;
 	}
 
@@ -703,7 +713,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == ELEMENT2 || parser->model == VEO20 ||
 		parser->model == A300 || parser->model == ZEN ||
 		parser->model == GEO || parser->model == GEO20 ||
-		parser->model == MANTA || parser->model == I300) {
+		parser->model == MANTA || parser->model == I300 ||
+		parser->model == I200) {
 		have_pressure = 0;
 	}
 
@@ -857,7 +868,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == VEO30 || parser->model == OC1A ||
 					parser->model == OC1B || parser->model == OC1C ||
 					parser->model == OCI || parser->model == A300 ||
-					parser->model == I450T || parser->model == I300) {
+					parser->model == I450T || parser->model == I300 ||
+					parser->model == I200) {
 					temperature = data[offset + 3];
 				} else if (parser->model == OCS || parser->model == TX1) {
 					temperature = data[offset + 1];
@@ -867,7 +879,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == XPAIR) {
 					temperature = ((data[offset + 7] & 0xF0) >> 4) | ((data[offset + 7] & 0x0C) << 2) | ((data[offset + 5] & 0x0C) << 4);
 				} else if (parser->model == A300CS || parser->model == VTX ||
-					parser->model == I750TC) {
+					parser->model == I750TC || parser->model == PROPLUSX) {
 					temperature = data[offset + 11];
 				} else {
 					unsigned int sign;
@@ -907,7 +919,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == VISION || parser->model == XPAIR)
 					pressure = (((data[offset + 0] & 0x03) << 8) + data[offset + 1]) * 5;
 				else if (parser->model == TX1 || parser->model == A300CS ||
-					parser->model == VTX || parser->model == I750TC)
+					parser->model == VTX || parser->model == I750TC ||
+					parser->model == PROPLUSX)
 					pressure = array_uint16_le (data + offset + 4);
 				else
 					pressure -= data[offset + 1];
@@ -924,7 +937,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				parser->model == VEO30 || parser->model == OC1A ||
 				parser->model == OC1B || parser->model == OC1C ||
 				parser->model == OCI || parser->model == A300 ||
-				parser->model == I450T || parser->model == I300)
+				parser->model == I450T || parser->model == I300 ||
+				parser->model == I200)
 				depth = (data[offset + 4] + (data[offset + 5] << 8)) & 0x0FFF;
 			else if (parser->model == ATOM1)
 				depth = data[offset + 3] * 16;
@@ -954,7 +968,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			unsigned int have_deco = 0;
 			unsigned int decostop = 0, decotime = 0;
 			if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I450T || parser->model == I750TC) {
+				parser->model == I450T || parser->model == I750TC ||
+				parser->model == PROPLUSX) {
 				decostop = (data[offset + 15] & 0x70) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x03FF;
 				have_deco = 1;
@@ -971,6 +986,12 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				decostop = (data[offset + 5] & 0xF0) >> 4;
 				decotime = array_uint16_le(data + offset + 4) & 0x03FF;
 				have_deco = 1;
+			} else if (parser->model == I200 || parser->model == I300 ||
+				parser->model == OC1A || parser->model == OC1B ||
+				parser->model == OC1C || parser->model == OCI) {
+				decostop = (data[offset + 7] & 0xF0) >> 4;
+				decotime = array_uint16_le(data + offset + 6) & 0x0FFF;
+				have_deco = 1;
 			}
 			if (have_deco) {
 				if (decostop) {
@@ -989,7 +1010,9 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			if (parser->model == ATOM31) {
 				rbt = array_uint16_le(data + offset + 6) & 0x01FF;
 				have_rbt = 1;
-			} else if (parser->model == I450T) {
+			} else if (parser->model == I450T || parser->model == OC1A ||
+				parser->model == OC1B || parser->model == OC1C ||
+				parser->model == OCI || parser->model == PROPLUSX) {
 				rbt = array_uint16_le(data + offset + 8) & 0x01FF;
 				have_rbt = 1;
 			} else if (parser->model == VISION || parser->model == XPAIR ||
@@ -1002,6 +1025,20 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				if (callback) callback (DC_SAMPLE_RBT, sample, userdata);
 			}
 
+			// Bookmarks
+			unsigned int have_bookmark = 0;
+			if (parser->model == OC1A || parser->model == OC1B ||
+				parser->model == OC1C || parser->model == OCI) {
+				have_bookmark = data[offset + 12] & 0x80;
+			}
+			if (have_bookmark) {
+				sample.event.type = SAMPLE_EVENT_BOOKMARK;
+				sample.event.time = 0;
+				sample.event.flags = 0;
+				sample.event.value = 0;
+				if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
+			}
+
 			count++;
 			complete = 1;
 		}
