@@ -32,6 +32,7 @@ extern "C" {
 
 #define ID_SERIAL   0x8010
 #define ID_FIRMWARE 0x8011
+#define ID_RDBI     0x8021
 #define ID_HARDWARE 0x8050
 
 #define PREDATOR 2
@@ -40,6 +41,7 @@ extern "C" {
 #define PERDIX   5
 #define PERDIXAI 6
 #define NERD2    7
+#define TERIC    8
 
 #define NSTEPS    10000
 #define STEP(i,n) ((NSTEPS * (i) + (n) / 2) / (n))
@@ -52,6 +54,9 @@ typedef struct shearwater_common_device_t {
 dc_status_t
 shearwater_common_setup (shearwater_common_device_t *device, dc_context_t *context, dc_iostream_t *iostream);
 
+dc_status_t
+shearwater_common_command (shearwater_common_device_t *device, const unsigned char input[], unsigned int isize);
+
 dc_status_t
 shearwater_common_transfer (shearwater_common_device_t *device, const unsigned char input[], unsigned int isize, unsigned char output[], unsigned int osize, unsigned int *actual);
 
